# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel backend DBus service."""

# pylint: disable=C0103
# DBus signals have CamelCased names

import dbus
import ubuntu_sso

from twisted.internet.defer import inlineCallbacks
from ubuntuone.devtools.testcase import DBusTestCase as TestCase
# pylint: disable=E0611, F0401
from ubuntuone.syncdaemon import dbus_interface as sd_dbus_iface
# pylint: enable=E0611, F0401

from ubuntuone.controlpanel import dbus_service
from ubuntuone.controlpanel import dbus_client


SAMPLE_CREDS = {
    "token": "ABCDEF12345678",
    "access_token": "DEADCAFE2010",
}

SAMPLE_LIMITS = {
    "upload": "999",
    "download": "838",
}

# pylint: disable=C0322
# pylint, you have to go to decorator's school


class MockDBusSSOService(dbus.service.Object):
    """A mock object that mimicks ussoc."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Get creds from the keyring, login/register if needed."""
        self.CredentialsFound(app_name, SAMPLE_CREDS)

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sa{ss}")
    def CredentialsFound(self, app_name, credentials):
        """Credentials were finally found."""


class MockDBusSSOServiceOther(dbus.service.Object):
    """A mock object that mimicks ussoc."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Get creds from the keyring, login/register if needed."""
        self.CredentialsFound("wrong app name", {})
        self.CredentialsFound(app_name, SAMPLE_CREDS)

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sa{ss}")
    def CredentialsFound(self, app_name, credentials):
        """Credentials were finally found."""


class MockDBusSSOClientOtherFailing(dbus.service.Object):
    """A mock object that mimicks ussoc."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Get creds from the keyring, login/register if needed."""
        self.CredentialsError("wrong app", "error message", "error details")
        self.CredentialsFound(app_name, SAMPLE_CREDS)

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sa{ss}")
    def CredentialsFound(self, app_name, credentials):
        """Credentials were finally found."""

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sss")
    def CredentialsError(self, app_name, error_message, detailed_error):
        """Some error happened and credentials were not found."""


class MockDBusSSOClientFailing(dbus.service.Object):
    """A mock object that mimicks ussoc but fails."""

    @dbus.service.method(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         in_signature="sssx")
    def login_or_register_to_get_credentials(self, app_name, tcurl, hlp, wid):
        """Fail while trying to get creds from the keyring."""
        self.CredentialsError(app_name, "error message", "error details")

    @dbus.service.signal(dbus_interface=ubuntu_sso.DBUS_IFACE_CRED_NAME,
                         signature="sss")
    def CredentialsError(self, app_name, error_message, detailed_error):
        """Some error happened and credentials were not found."""


class MockDBusNoMethods(dbus.service.Object):
    """A mock that fails at the DBus layer (because it's got no methods!)."""


class MockDBusSyncDaemon(dbus.service.Object):
    """A mock object that mimicks syncdaemon."""
    throttling = False
    limits = {"download": -1, "upload": -1}

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME,
                         in_signature="", out_signature="a{si}")
    def get_throttling_limits(self):
        """Return the throttling dict."""
        return self.limits

    @dbus.service.method(in_signature="ii",
               dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def set_throttling_limits(self, download, upload):
        """Set the throttling dict."""
        self.limits["download"] = download
        self.limits["upload"] = upload

    @dbus.service.method(in_signature="", out_signature="b",
              dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def bandwidth_throttling_enabled(self):
        """Get the state of bw throttling."""
        return self.throttling

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def enable_bandwidth_throttling(self):
        """Enable bw throttling."""
        self.throttling = True

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def disable_bandwidth_throttling(self):
        """Disable bw throttling."""
        self.throttling = False


class TestDBusException(Exception):
    """A DBus exception to be used in tests."""


class MockDBusSyncDaemonFailing(dbus.service.Object):
    """A mock object that mimicks syncdaemon but fails."""

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME,
                         in_signature="", out_signature="a{si}")
    def get_throttling_limits(self):
        """Fail while getting the throttling dict."""
        raise TestDBusException()

    @dbus.service.method(in_signature="ii",
               dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def set_throttling_limits(self, download, upload):
        """Fail while setting the throttling dict."""
        raise TestDBusException()

    @dbus.service.method(in_signature="", out_signature="b",
              dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def bandwidth_throttling_enabled(self):
        """Get the state of bw throttling."""
        raise TestDBusException()

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def enable_bandwidth_throttling(self):
        """Enable bw throttling."""
        raise TestDBusException()

    @dbus.service.method(dbus_interface=sd_dbus_iface.DBUS_IFACE_CONFIG_NAME)
    def disable_bandwidth_throttling(self):
        """Disable bw throttling."""
        raise TestDBusException()


class DBusClientTestCase(TestCase):
    """Test for the dbus client."""

    timeout = 5
    mock = None

    def setUp(self):
        super(DBusClientTestCase, self).setUp()
        dbus_service.init_mainloop()

    def register_mockserver(self, bus_name, object_path, object_class):
        """The mock service is registered on the DBus."""
        session_bus = dbus.SessionBus()
        name = session_bus.request_name(bus_name,
                                        dbus.bus.NAME_FLAG_DO_NOT_QUEUE)
        self.assertNotEqual(name, dbus.bus.REQUEST_NAME_REPLY_EXISTS)
        self.mock = object_class(object_path=object_path, conn=session_bus)
        self.addCleanup(self.mock.remove_from_connection)

    @inlineCallbacks
    def test_get_credentials_ok(self):
        """Test the success case for get_credentials."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH, MockDBusSSOService)
        creds = yield dbus_client.get_credentials()
        self.assertEqual(creds, SAMPLE_CREDS)

    @inlineCallbacks
    def test_get_credentials_other(self):
        """Creds for other apps are ignored."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusSSOServiceOther)
        creds = yield dbus_client.get_credentials()
        self.assertEqual(creds, SAMPLE_CREDS)

    @inlineCallbacks
    def test_get_credentials_error(self):
        """Test what happens when the creds can't be retrieved."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusSSOClientFailing)

        yield self.assertFailure(dbus_client.get_credentials(),
                                 dbus_client.CredentialsError)

    @inlineCallbacks
    def test_get_credentials_other_error(self):
        """Other creds err before ours are retrieved."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusSSOClientOtherFailing)

        creds = yield dbus_client.get_credentials()
        self.assertEqual(creds, SAMPLE_CREDS)

    @inlineCallbacks
    def test_get_credentials_dbus_error(self):
        """Test what happens when there's a DBus error."""
        self.register_mockserver(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CRED_PATH,
                                 MockDBusNoMethods)

        yield self.assertFailure(dbus_client.get_credentials(),
                                 dbus.DBusException)

    @inlineCallbacks
    def test_throttle_limits_returned(self):
        """When SD returns the limits, they are handled."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemon)
        limits = yield dbus_client.get_throttling_limits()
        self.assertEqual(limits, MockDBusSyncDaemon.limits)

    @inlineCallbacks
    def test_getting_throttle_limits_throws_an_error(self):
        """Handle DBus error when getting the limits."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemonFailing)
        yield self.assertFailure(dbus_client.get_throttling_limits(),
                                 dbus.DBusException)

    @inlineCallbacks
    def test_throttle_limits_set(self):
        """Setting the limits."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemon)
        yield dbus_client.set_throttling_limits(SAMPLE_LIMITS)
        expected_result = {
            "download": int(SAMPLE_LIMITS["download"]),
            "upload": int(SAMPLE_LIMITS["upload"]),
        }
        self.assertEqual(self.mock.limits, expected_result)

    @inlineCallbacks
    def test_setting_throttle_limits_throws_an_error(self):
        """Handle DBus error when setting the limits."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemonFailing)
        d = dbus_client.set_throttling_limits(SAMPLE_LIMITS)
        yield self.assertFailure(d, dbus.DBusException)

    @inlineCallbacks
    def test_throttle_get(self):
        """Getting the throttling state."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemon)
        self.mock.throttling = False
        throttling = yield dbus_client.bandwidth_throttling_enabled()
        self.assertEqual(throttling, False)
        self.mock.throttling = True
        throttling = yield dbus_client.bandwidth_throttling_enabled()
        self.assertEqual(throttling, True)

    @inlineCallbacks
    def test_throttle_get_throws_an_error(self):
        """Handle DBus error when getting the throttling state."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemonFailing)
        d = dbus_client.bandwidth_throttling_enabled()
        yield self.assertFailure(d, dbus.DBusException)

    @inlineCallbacks
    def test_throttle_enable(self):
        """Enabling bw throttling."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemon)
        self.mock.throttling = False
        yield dbus_client.enable_bandwidth_throttling()
        self.assertEqual(self.mock.throttling, True)

    @inlineCallbacks
    def test_throttle_enable_throws_an_error(self):
        """Handle DBus error when enabling throttling."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemonFailing)
        d = dbus_client.enable_bandwidth_throttling()
        yield self.assertFailure(d, dbus.DBusException)

    @inlineCallbacks
    def test_throttle_disable(self):
        """Disabling bw throttling."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemon)
        self.mock.throttling = True
        yield dbus_client.disable_bandwidth_throttling()
        self.assertEqual(self.mock.throttling, False)

    @inlineCallbacks
    def test_throttle_disable_throws_an_error(self):
        """Handle DBus error when disabling throttling."""
        self.register_mockserver(sd_dbus_iface.DBUS_IFACE_NAME,
                                 "/config", MockDBusSyncDaemonFailing)
        d = dbus_client.disable_bandwidth_throttling()
        yield self.assertFailure(d, dbus.DBusException)
