# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A backend for the Ubuntu One Control Panel."""

from twisted.internet.defer import inlineCallbacks, returnValue

from ubuntuone.controlpanel import dbus_client
from ubuntuone.controlpanel.webclient import WebClient

ACCOUNT_API = "account/"
QUOTA_API = "quota/"
DEVICES_API = "1.0/devices/"
DEVICE_REMOVE_API = "1.0/devices/remove/%s/%s"
DEVICE_TYPE_PHONE = "Phone"
DEVICE_TYPE_COMPUTER = "Computer"
UPLOAD_KEY = "max_upload_speed"
DOWNLOAD_KEY = "max_download_speed"


class ControlBackend(object):
    """The control panel backend."""

    def __init__(self):
        """Initialize the webclient."""
        self.wc = WebClient(dbus_client.get_credentials)

    @inlineCallbacks
    def get_token(self):
        """Return the token from the credentials"""
        credentials = yield dbus_client.get_credentials()
        returnValue(credentials["token"])

    @inlineCallbacks
    def account_info(self):
        """Get the user account info."""
        result = {}

        account_info = yield self.wc.call_api(ACCOUNT_API)
        result["type"] = account_info["subscription"]["description"]
        result["name"] = account_info["nickname"]
        result["email"] = account_info["email"]

        quota_info = yield self.wc.call_api(QUOTA_API)
        result["quota_total"] = str(quota_info["total"])
        result["quota_used"] = str(quota_info["used"])

        returnValue(result)

    @inlineCallbacks
    def devices_info(self):
        """Get the user devices info."""
        result = []
        this_token = yield self.get_token()
        limit_bw = yield dbus_client.bandwidth_throttling_enabled()
        limits = yield dbus_client.get_throttling_limits()

        devices = yield self.wc.call_api(DEVICES_API)
        for d in devices:
            di = {}
            result.append(di)
            di["type"] = d["kind"]
            di["name"] = d["description"]
            di["configurable"] = "0"
            if di["type"] == DEVICE_TYPE_COMPUTER:
                di["device_id"] = di["type"] + d["token"]
                if d["token"] == this_token:
                    di["configurable"] = "1"
                    di["limit_bandwidth"] = "1" if limit_bw else "0"
                    upload = limits["upload"]
                    download = limits["download"]
                    di[UPLOAD_KEY] = str(upload)
                    di[DOWNLOAD_KEY] = str(download)
            if di["type"] == DEVICE_TYPE_PHONE:
                di["device_id"] = di["type"] + str(d["id"])

            # date_added is not in the webservice yet (LP: #673668)
            # di["date_added"] = ""

            # missing values (LP: #673668)
            # di["available_services"] = ""
            # di["enabled_services"] = ""

        returnValue(result)

    def type_n_id(self, device_id):
        """Return the device type and id, as used by the /devices api."""
        if device_id.startswith(DEVICE_TYPE_COMPUTER):
            return DEVICE_TYPE_COMPUTER, device_id[8:]
        if device_id.startswith(DEVICE_TYPE_PHONE):
            return DEVICE_TYPE_PHONE, device_id[5:]
        return "No device", device_id

    @inlineCallbacks
    def change_device_settings(self, device_id, settings):
        """Change the settings for the given device."""
        dtype, did = self.type_n_id(device_id)
        local_token = yield self.get_token()
        is_local = (dtype == DEVICE_TYPE_COMPUTER and did == local_token)

        if is_local and "limit_bandwidth" in settings:
            limit_bw = settings["limit_bandwidth"]
            if limit_bw == "0":
                yield dbus_client.disable_bandwidth_throttling()
            if limit_bw == "1":
                yield dbus_client.enable_bandwidth_throttling()

        if is_local and (UPLOAD_KEY in settings or
                         DOWNLOAD_KEY in settings):
            current_limits = yield dbus_client.get_throttling_limits()
            limits = {
                "download": current_limits["download"],
                "upload": current_limits["upload"],
            }
            if UPLOAD_KEY in settings:
                limits["upload"] = settings[UPLOAD_KEY]
            if DOWNLOAD_KEY in settings:
                limits["download"] = settings[DOWNLOAD_KEY]
            dbus_client.set_throttling_limits(limits)

        # still pending: more work on the settings dict (LP: #673674)
        returnValue(device_id)

    @inlineCallbacks
    def remove_device(self, device_id):
        """Remove a device's tokens from the sso server."""
        dtype, did = self.type_n_id(device_id)
        api = DEVICE_REMOVE_API % (dtype, did)
        yield self.wc.call_api(api)
        returnValue(device_id)

    def file_sync_status(self):
        """Return the status of the file sync service."""
        # still pending (LP: #673670)
        returnValue((True, "Synchronizing"))

    def query_bookmark_extension(self):
        """True if the bookmark extension has been installed."""
        # still pending (LP: #673672)
        returnValue(False)

    def install_bookmarks_extension(self):
        """Install the extension to sync bookmarks."""
        # still pending (LP: #673673)
        returnValue(None)
