# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the control panel backend."""

import simplejson

from twisted.internet import defer
from twisted.internet.defer import inlineCallbacks
from twisted.trial.unittest import TestCase

from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.backend import (ACCOUNT_API, QUOTA_API,
                                            DEVICES_API, DEVICE_REMOVE_API)
from ubuntuone.controlpanel.webclient import WebClientError

SAMPLE_CREDENTIALS = {"token": "ABC1234DEF"}

SAMPLE_ACCOUNT_JSON = """
{
    "username": "andrewpz",
    "openid": "https://login.launchpad.net/+id/abcdefg",
    "first_name": "Andrew P.",
    "last_name": "Zoilo",
    "couchdb": {
        "host": "https://couchdb.one.ubuntu.com",
        "root": "https://couchdb.one.ubuntu.com/u/abc/def/12345",
        "dbpath": "u/abc/def/12345"
    },
    "couchdb_root": "https://couchdb.one.ubuntu.com/u/abc/def/12345",
    "email": "andrewpz@protocultura.net",
    "nickname": "Andrew P. Zoilo",
    "id": 12345,
    "subscription": {
        "upgrade_available": false,
        "description": "Paid Plan, 50 GB of storage",
        "trial": false,
        "started": "2010-03-24T18:38:38Z",
        "is_paid": true,
        "expires": null,
        "qty": 1,
        "price": 0.0,
        "currency": null,
        "id": 654321,
        "name": "50 GB"
    }
}
"""

SAMPLE_QUOTA_JSON = """
{
    "total": 53687091200,
    "used": 2350345156
}
"""

EXPECTED_ACCOUNT_INFO = {
    "quota_used": "2350345156",
    "quota_total": "53687091200",
    "type": "Paid Plan, 50 GB of storage",
    "name": "Andrew P. Zoilo",
    "email": "andrewpz@protocultura.net",
}

SAMPLE_DEVICES_JSON = """
[
    {
        "token": "ABCDEF01234token",
        "description": "Ubuntu One @ darkstar",
        "kind": "Computer"
    },
    {
        "token": "ABC1234DEF",
        "description": "Ubuntu One @ localhost",
        "kind": "Computer"
    },
    {
        "kind": "Phone",
        "description": "Nokia E65",
        "id": 1000
    }
]
"""

EXPECTED_DEVICES_INFO = [
    {
        "device_id": "ComputerABCDEF01234token",
        "name": "Ubuntu One @ darkstar",
        "type": "Computer",
        "configurable": "0",
    },
    {
        'configurable': '1',
        'device_id': 'ComputerABC1234DEF',
        'limit_bandwidth': "0",
        'max_download_speed': '-1',
        'max_upload_speed': '-1',
        'name': 'Ubuntu One @ localhost',
        'type': 'Computer'
    },
    {
        "device_id": "Phone1000",
        "name": "Nokia E65",
        "type": "Phone",
        "configurable": "0",
    },
]


class MockWebClient(object):
    """A mock webclient."""
    failure = False
    results = {}

    def __init__(self, get_credentials):
        """Initialize this mock instance."""
        self.get_credentials = get_credentials

    def call_api(self, method):
        """Get a given url from the webservice."""
        if self.failure:
            return defer.fail(WebClientError(self.failure))
        else:
            result = simplejson.loads(self.results[method])
            return defer.succeed(result)


class MockDBusClient(object):
    """A mock dbus_client module."""
    creds = SAMPLE_CREDENTIALS
    throttling = False
    limits = {"download": -1, "upload": -1}

    def get_credentials(self):
        """Return the mock credentials."""
        return defer.succeed(self.creds)

    def get_throttling_limits(self):
        """Return the sample speed limits."""
        return self.limits

    def set_throttling_limits(self, limits):
        """Set the sample speed limits."""
        self.limits["download"] = int(limits["download"])
        self.limits["upload"] = int(limits["upload"])

    def bandwidth_throttling_enabled(self):
        """Return the state of throttling."""
        return self.throttling

    def enable_bandwidth_throttling(self):
        """Enable bw throttling."""
        self.throttling = True

    def disable_bandwidth_throttling(self):
        """Disable bw throttling."""
        self.throttling = False


class BackendBasicTestCase(TestCase):
    """Simple tests for the backend."""
    timeout = 3

    def setUp(self):
        self.patch(backend, "WebClient", MockWebClient)
        self.patch(backend, "dbus_client", MockDBusClient())
        self.local_token = "Computer" + SAMPLE_CREDENTIALS["token"]

    def test_backend_creation(self):
        """The backend instance is successfully created."""
        be = backend.ControlBackend()
        self.assertEqual(be.wc.__class__, MockWebClient)

    @inlineCallbacks
    def test_account_info(self):
        """The account_info method exercises its callback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.results[ACCOUNT_API] = SAMPLE_ACCOUNT_JSON
        be.wc.results[QUOTA_API] = SAMPLE_QUOTA_JSON
        result = yield be.account_info()
        self.assertEqual(result, EXPECTED_ACCOUNT_INFO)

    @inlineCallbacks
    def test_account_info_fails(self):
        """The account_info method exercises its errback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.failure = 404
        yield self.assertFailure(be.account_info(), WebClientError)

    @inlineCallbacks
    def test_devices_info(self):
        """The devices_info method exercises its callback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.results[DEVICES_API] = SAMPLE_DEVICES_JSON
        result = yield be.devices_info()
        self.assertEqual(result, EXPECTED_DEVICES_INFO)

    @inlineCallbacks
    def test_devices_info_fails(self):
        """The devices_info method exercises its errback."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.failure = 404
        yield self.assertFailure(be.devices_info(), WebClientError)

    @inlineCallbacks
    def test_remove_device(self):
        """The remove_device method calls the right api."""
        be = backend.ControlBackend()
        dtype, did = "Computer", "SAMPLE-TOKEN"
        device_id = dtype + did
        apiurl = DEVICE_REMOVE_API % (dtype, did)
        # pylint: disable=E1101
        be.wc.results[apiurl] = SAMPLE_DEVICES_JSON
        result = yield be.remove_device(device_id)
        self.assertEqual(result, device_id)

    @inlineCallbacks
    def test_remove_device_fails(self):
        """The remove_device method fails as expected."""
        be = backend.ControlBackend()
        # pylint: disable=E1101
        be.wc.failure = 404
        yield self.assertFailure(be.devices_info(), WebClientError)

    @inlineCallbacks
    def test_change_limit_bandwidth(self):
        """The device settings are updated."""
        be = backend.ControlBackend()
        backend.dbus_client.throttling = False
        yield be.change_device_settings(self.local_token,
                                        {"limit_bandwidth": "1"})
        self.assertEqual(backend.dbus_client.throttling, True)
        yield be.change_device_settings(self.local_token,
                                        {"limit_bandwidth": "0"})
        self.assertEqual(backend.dbus_client.throttling, False)

    @inlineCallbacks
    def test_change_upload_speed_limit(self):
        """The device settings are updated."""
        be = backend.ControlBackend()
        backend.dbus_client.limits = {"download": -1, "upload": -1}
        yield be.change_device_settings(self.local_token,
                                        {"max_upload_speed": "1111"})
        self.assertEqual(backend.dbus_client.limits["upload"], 1111)
        self.assertEqual(backend.dbus_client.limits["download"], -1)

    @inlineCallbacks
    def test_change_download_speed_limit(self):
        """The device settings are updated."""
        be = backend.ControlBackend()
        backend.dbus_client.limits = {"download": -1, "upload": -1}
        yield be.change_device_settings(self.local_token,
                                        {"max_download_speed": "99"})
        self.assertEqual(backend.dbus_client.limits["upload"], -1)
        self.assertEqual(backend.dbus_client.limits["download"], 99)

    @inlineCallbacks
    def test_changing_settings_for_wrong_id_has_no_effect(self):
        """If the id is wrong, no settings are changed."""
        be = backend.ControlBackend()
        backend.dbus_client.throttling = False
        backend.dbus_client.limits = {"download": -1, "upload": -1}
        new_settings = {
            "max_download_speed": "99",
            "max_upload_speed": "99",
            "limit_bandwidth": "1",
        }
        yield be.change_device_settings("wrong token!", new_settings)
        self.assertEqual(backend.dbus_client.throttling, False)
        self.assertEqual(backend.dbus_client.limits["upload"], -1)
        self.assertEqual(backend.dbus_client.limits["download"], -1)

    @inlineCallbacks
    def test_get_token(self):
        """The get_token method returns the right token."""
        be = backend.ControlBackend()
        token = yield be.get_token()
        self.assertEqual(token, SAMPLE_CREDENTIALS["token"])
